; ds1620_lib-0.1.asm                                      (09/1999)
; Routines that control the Dallas Semiconductor DS1620 temperature 
; chip.  (Useable but not quite complete)
;
;--- REGISTER DEFINITIONS (^) = requires r16-31
; ds1620_tx (^)   = loaded with the command byte sent to the '1620
; ds1620_rx       = holds the low byte (value) of temp. conversion
;                   and results of '1620 commands that return data
;                   The N flag is set to indicate negative temps
; ds1620_tmp (^)  = a scratchpad register
; ds1620_tmp2 (^) = another scratch register
;
;
;--- I/O DEFINITIONS
; 3 of the AVR's i/o pins are required. All MUST be defined as pins 
; on portd.
;   rst = the '1620's RESET line (always an output)
;   clk  =  '     '    clock  "   (   "    "    "  )
;   dq   = data line (can be either input or output)
;
;
;--- CALLABLE MACROS
;_______________NAME _ ARGUMENT___   RESULT_________________________
;          read_temp      n/a        temp value in 1620_rx, sign in N 
;           write_TH
;           write_TL
;            read_TH      n/a
;            read_TL      n/a
;       read_counter
;         read_slope
;    start_convert_T      n/a
;     stop_convert_T      n/a
;       write_config <config_byte>
;        read_config      n/a
;
; --- ds1620_lib description END ----------------------------------------
.NOLIST


.macro start_convert_T ; starts temperature conversion
                         ldi   ds1620_tx, 0xee
                         rcall _1620_write
                         rcall _1620_reset
                         .endmacro


.macro stop_convert_T ; stops temperature conversion
                        ldi   ds1620_tx, 0x22
                        rcall _1620_write
                        rcall _1620_reset
                        .endmacro


.macro write_config ; writes config reg in '1620
                    ; @0 contains the config byte
                      ldi   ds1620_tx, 0x0c
                      rcall _1620_write
                      ldi   ds1620_tx, @0
                      rcall _1620_write
                      rcall _1620_reset
                      .endmacro


.macro read_temp ; reads the temperature
                 ; 1620_rx holds the value, N flag in SREG holds the sign
                   ldi   ds1620_tx, 0xaa
                   rcall _1620_write
                   rcall _1620_read             ; here's the temp
                   mov   ds1620_tx, ds1620_rx   ; store it temporarily
                   rcall _1620_read             ; get the sign bit
                   rcall _1620_reset
                   cln                          ; clear N (negative) flag
                   sbrc  ds1620_rx, 0           ; skip if bit 0 cleared
                   sen                          ; set N flag
                   mov   ds1620_rx, ds1620_tx   ; restore saved temp value
                   .endmacro



_1620_write:; Write a data byte to the '1620
            ; exits with CLK HIGH and DQ set to output
              sbi  DDRD, dq      ; set the DQ line to an output
              ldi  ds1620_tmp, 8 ; load the tmp reg with a bit count
_write_loop:  cbi  PORTD, clk    ; set clk LOW
              ror  ds1620_tx     ; shift right to put next bit in carry
              brcs _send_1       ; is it a "1"
              cbi  PORTD, dq     ; no, set DQ bit LOW
              rjmp _cont         ; and continue
_send_1:      sbi  PORTD, dq     ; set DQ bit HIGH
_cont:        nop                ; wait
              nop                ;      a 
              nop                ;      little
              sbi  PORTD, clk    ; set clk HIGH
              nop                ;     and leave
              nop                ;     it that way
              nop                ;     for awhile
              dec  ds1620_tmp    ; do the write_loop
              brne _write_loop   ;    8 times
              ret                ; done


_1620_read:; Read a data byte from the '1620
           ; exits with CLK HIGH and DQ set to input
             cbi  DDRD, dq          ; set the DQ line to an input
             ldi  ds1620_tmp, 8     ; load the tmp reg with a bit count
_read_loop:  cbi  PORTD, clk        ; set clk LOW
             nop                    ; wait
             nop                    ;      a
             nop                    ;        while
             lsr  ds1620_rx         ; shift right to make room for next bit
             in   ds1620_tmp2, PIND ; read the state of DQ
             bst  ds1620_tmp2, DQ   ;      into the T bit
             bld  ds1620_rx, 7      ; copy it into MSB of the rx reg
             sbi  PORTD, clk        ; set clk HIGH
             nop                    ;     and leave
             nop                    ;     it that way
             nop                    ;     for awhile
             dec  ds1620_tmp        ; do the read_loop
             brne _read_loop        ;    8 times
             ret                    ; done


_1620_reset: ; Sends a reset pulse to the '1620
               cbi  PORTD, rst   ; clear /rst bit
               nop               ; wait
               nop               ;  a
               nop               ;  little
               sbi  PORTD, rst   ; set /rst bit
               ret               ; return


;EOF: ds1620_lib-0.1.asm
.LIST
