; thermo-1.0.asm;  Wireless Digital Thermometer Program    Oct 1999
; Read temperature from ds1620 and output tones


;--- REGISTER DEFINITIONS ---------------------------------------------
.def   tick_count   = r0  ; The low-level tick counter, controls all timing
.def   tick_temp    = r1  ; Destructive copy of tick_count
.def   hp_temp      = r2  ;   "          "   "  half_period
.def   cyc_l_temp   = r3  ;   "          "   "  cycles_low
.def   cyc_h_temp   = r4  ;   "          "   "  cycles_high
.def   half_period  = r16 ; The half period time, in ticks
.def   cycles_l     = r17 ; Number of cycles, low byte
.def   cycles_h     = r18 ; Number of cycles, high byte
.def   ds1620_tx    = r19 ; Used by 1620_lib
.def   ds1620_rx    = r20 ;  "   "    "
.def   ds1620_tmp   = r21 ;  "   "    "
.def   ds1620_tmp2  = r22 ;  "   "    "
.def   sleep_ctr    = r23 ; Sleep counter
.def   digit        = r24 ; Holds the current BCD digit (0-9) to xmit
.def   tmp          = r30 ; temp storage
.def   rx_tmp       = r31 ; temp storage


;--- PORT DEFINITIONS -------------------------------------------------
; Port D, bit 6 = audio modulation output
;         bit 5 = transmitter keying output,  0 = not transmitting
                ; pd76543210
.equ portd_defs = 0b01111010
.equ audio      = 6 ; audio out line
.equ xmit       = 5 ; transmitter control bit (0 = transmit)
.equ rst        = 4 ; the 1620's reset line
.equ clk        = 3 ; the 1620's clock line
.equ dq         = 2 ; the 1620's bidirectional data line
.equ tirq_test  = 1 ; for interrupt testing

; Port B, set to outputs
                ; pb76543210
.equ portb_defs = 0b11111111


;--- INTERRUPT VECTORS ------------------------------------------------
          .org  0
   RESET: rjmp  main   ; RESET vector
EXT_INT0: rjmp  INTR0  ; Int 0 handler
TIM0_OVF: rjmp  TIMR0  ; Timer 0 overflow int handler
ANA_COMP: rjmp  COMP   ; Analog comparator int handler


.include "../include/1200.def"
.include "tone_lib-0.1.asm"
.include "ds1620_lib-0.1.asm"


;--- THE PROGRAM ------------------------------------------------------
main:     ldi   tmp, portd_defs   ; get port D direction definition
          out   DDRD, tmp         ;     put it in DDRD
          ldi   tmp, portb_defs   ; get port B direction definition
          out   DDRB, tmp         ;     put it in DDRB
          sbi   PORTD, xmit       ; set xmit bit to 1 (turn off transmitter)
          ldi   tmp, 15           ; set up the tick count for about 100uS
          mov   tick_count, tmp   ;     @ 500KHz (more like 98uS, but close)
          sbi   ACSR, ACD         ; turn off the analog comp. to save power
          ldi   tmp,  0x20
          out   MCUCR, tmp        ; enable sleep mode
          write_config 0x0b       ; set the '1620 for cpu and single shot mode
          start_convert_t         ; start the 1620's temp conversion
          ldi   sleep_ctr, 30     ; load the sleep counter w/15s of sleep
          ldi   tmp, 0x05
          out   TCCR0, tmp        ; set the t/c prescaler for ck/1024
          ldi   tmp, 0x02
          out   TIMSK, tmp        ; enable timer interrupt
          sei                     ; enable global interrupt
snooze:   ldi   tmp, 13
          out   TCNT0, tmp        ; load the t/c for 500mS sleep time
          sleep                   ;   / sleep \
                                  ;   |  for  |  (actually, about 499.7mS)
                                  ;   \ 500mS /
awake:    dec   sleep_ctr         ; decrement the sleep counter
          brne  snooze            ; sleep counter =0? no, goto "snooze"

          clr   tmp
          out   TCCR0, tmp        ; stop the t/c (no ints while sending)
          ldi   sleep_ctr, 30     ; load the sleep counter with another 15s
          cbi   PORTD, xmit       ; Turn on transmitter
          read_temp               ; get the temp from the '1620
          mov   rx_tmp, ds1620_rx ; make a copy of the temperature
          brpl  hundreds          ; skip ahead if temperature is >0
          ldi   digit, 0          ; output a leading "0" if the
          rcall send              ; temp. is negative
          neg   rx_tmp            ; convert temperature from two's comp

hundreds: lsr   rx_tmp            ; shift right 1 bit, rx_tmp now 0-127
          cpi   rx_tmp, 100       ; is temperature > 100?
          brlo  tens              ; skip ahead if not
          ldi   digit, 1
          rcall send              ; else, output a "1"
          subi  rx_tmp, 100       ; and subtract the 100's digit

tens:     ldi   tmp, 90           ; load a tmp reg with 90
          ldi   digit, 9          ; 
loop:     cp    rx_tmp, tmp       ; tens part of rx_tmp > tmp?
          brsh  ones              ; yes, move on to the ones
          subi  tmp, 10           ; no, subtract 10 from tmp
          subi  digit, 1          ;     and 1 from digit
          rjmp  loop              ;     and keep trying

ones:     rcall send              ; first, send the tens digit
          sub   rx_tmp, tmp       ; subtract off the tens digit
          mov   digit, rx_tmp     ; nothing left to do but
          rcall send              ;     send the ones digit

          ldi   digit, 0          ; preload a zero before we test
          sbrc  ds1620_rx, 0      ; skip next if temperature is xx.0
          ldi   digit, 5          ; otherwise, temp. was xx.5
          rcall send              ; send it

          pause 2500              ; t=0.25s
          sbi   PORTD, xmit       ; turn off transmitter
          start_convert_t         ; start the 1620's temp conversion
          ldi   tmp, 0x05
          out   TCCR0, tmp        ; restart the t/c
          rjmp  snooze            ; Do forever.


;--- SUBROUTINES ------------------------------------------------------

send: ; Outputs a tone sequence for a BCD digit (0-9) in the "digit" 
      ; register

            pause 10000       ; pause between chars
            tst   digit       ; special case, is digit 0?
            brne  not0        ; no, skip ahead
            sound 208, 6      ; send a "5" (208 cycles of 833.3Hz)
            pause 1250        ; space
            sound 208, 6      ; send another "5"   (5+5=(1)0)
            rjmp  done        ; 
not0:       cpi   digit, 5    ; Is the digit > 5
            brlo  lessthan5   ; no, it's 1-4
            sound 208, 6      ; yes, send a "5"
            pause 1250        ; space
            subi  digit, 5    ; and subtract off the amount we sent
lessthan5:  tst   digit       ; is digit 0
            breq  done        ; yes, we're done
            sound 125, 10     ; no, send a "1"  (125 cycles of 500Hz)
            pause 1250        ; send a "space"
            dec   digit       ; and subtract off 1
            rjmp  lessthan5   ; go back and check for 0 again
done:       ret               ; finished


;--- INTERRUPT SERVICE ROUTINES ---------------------------------------
INTR0:  reti ; (not used)

TIMR0:  ; sbi PORTD, tirq_test    ; output a pulse
        ; cbi PORTD, tirq_test    ; as a debugging aid
        reti                    ; then return

COMP:   reti ; (not used)


; EOF: thermo-1.0.asm
